/*---------------------------------------------------------------------------

  defmake.c v0.65                 Greg Roelofs                  18 July 1991

  This program is basically a database of possible or actual tokens which may
  or may not be predefined by various C compilers (preprocessors, actually).
  Defmake builds another program, defind.c ("defined"/"de-find":  a little
  geek humor there, ha ha).  That program in turn may be run to determine 
  which, if any, of the included tokens are defined on the given system.  It
  is rather a brute-force approach, but it gets the job done for quite a few 
  systems.

  The purpose behind this, btw, is to provide information which may be used
  to enhance portability and reduce Makefile complexity.  The specific moti-
  vation was Info-ZIP's work on portable zip/unzip programs.

  ---------------------------------------------------------------------------

  To compile:

    under Unix:  make
      (or "make defind")

    under VMS:  @make_defind

    under MS-DOS:  <cc> defmake.c -o defmake
                   defmake
                   <cc> defind.c -o defind
      ("<cc>" = cl, tcc, or gcc--for MSC, Turbo C, or GNU C, respectively)

  ---------------------------------------------------------------------------

  To use:

    under Unix and MS-DOS:
       defind "your system description" > yourfile.def

    under VMS:
       defind == "$diskname:[directory]defind.exe"
       define/user sys$output yourfile.def
       defind "your system description"

  ---------------------------------------------------------------------------*/

#include <stdio.h>
#include "tokens.h"   /* has tokens[], standard[], and DEFIND_VERSION */

char  filename[]="defind.c";

main()
{
    int   i, num_tokens;
    FILE  *outfile;


    if ((outfile = fopen(filename, "w")) == NULL) {
        fprintf(stderr, "defmake:  can't create %s\n", filename);
        exit(1);
    }

/*---------------------------------------------------------------------------
    Write main() function.
  ---------------------------------------------------------------------------*/

    fprintf(outfile, "/*---------------------------------------------------------------------------\n");
    fprintf(outfile, "  defind.c, version %s (automatically generated by defmake,\n", DEFIND_VERSION);
    fprintf(outfile, "  another fine product from your good buddies at Newtware Industries, Inc.)\n");
    fprintf(outfile, "  ---------------------------------------------------------------------------*/\n");
    fprintf(outfile, "\n#include <stdio.h>\n\n");
    fprintf(outfile, "#ifdef VAXC                          /* In VAX C, sizeof(integer constant)   */\n");
    fprintf(outfile, "#  define COMPARE(x,y) ((x) <= (y))  /*  is not necessarily sizeof(int); it  */\n");
    fprintf(outfile, "#else /* !VAXC */                    /*  is the smallest possible data size. */\n");
    fprintf(outfile, "#  define COMPARE(x,y) ((x) == (y))  /*  Thus sizeof(1) is 1; sizeof(300) is */\n");
    fprintf(outfile, "#endif /* ?VAXC */                   /*  2; sizeof(70000) is 4; etc.         */\n");
    fprintf(outfile, "\n\nmain(argc, argv)\n");
    fprintf(outfile, "    int  argc;\n    char *argv[];\n{\n");
    fprintf(outfile, "    printf(\"This information is from defind.c, version %s.\\n\");\n\n", DEFIND_VERSION);
    fprintf(outfile, "    without_param(argc, argv);\n");
    fprintf(outfile, "#if !defined(VAXC) && !defined(MSDOS)\n");
    fprintf(outfile, "    with_param();\n");
    fprintf(outfile, "#endif /* !VAXC && !MSDOS */\n");
    fprintf(outfile, "    other();\n    exit(0);\n}\n\n\n");

/*---------------------------------------------------------------------------
    Write without_param() function.
  ---------------------------------------------------------------------------*/

    fprintf(outfile, "without_param(argc, argv)\n");
    fprintf(outfile, "    int  argc;\n    char *argv[];\n{\n");
    fprintf(outfile, "    printf(\"\\nThe following tokens are predefined by the preprocessor/\");\n");
    fprintf(outfile, "    if (argc > 1)\n");
    fprintf(outfile, "        printf(\"compiler for\\n%%s:\\n\", argv[1]);\n");
    fprintf(outfile, "    else\n        printf(\"compiler:\\n\");\n\n");

    for (i = 0;  i < sizeof(tokens)/sizeof(char *);  ++i) {
        fprintf(outfile, "#ifdef %s\n", tokens[i]);
        fprintf(outfile, "    printf( COMPARE(sizeof(%s),sizeof(int)) ?\n", 
          tokens[i]);
        fprintf(outfile, "      \"%24s (%%d)\\n\" :\n", tokens[i]);
        fprintf(outfile, "      \"%24s (%%s)\\n\", %s);\n", tokens[i], 
          tokens[i]);
        fprintf(outfile, "#endif\n");
    }

    fprintf(outfile, "}\n\n\n");
    num_tokens = i;

/*---------------------------------------------------------------------------
    Write with_param() function (anybody besides Unix systems??).
  ---------------------------------------------------------------------------*/

    fprintf(outfile, "#if defined(unix)\n");
    fprintf(outfile, "#include <sys/param.h>\n\nwith_param()\n{\n");
    fprintf(outfile, "    printf(\"\\nThe following tokens are defined when <sys/param.h> is included:\\n\");\n\n");

    for (i = 0;  i < sizeof(tokens)/sizeof(char *);  ++i) {
        fprintf(outfile, "#ifdef %s\n", tokens[i]);
        fprintf(outfile, "    printf( COMPARE(sizeof(%s),sizeof(int)) ?\n", 
          tokens[i]);
        fprintf(outfile, "      \"%24s (%%d)\\n\" :\n", tokens[i]);
        fprintf(outfile, "      \"%24s (%%s)\\n\", %s);\n", tokens[i], 
          tokens[i]);
        fprintf(outfile, "#endif\n");
    }

    fprintf(outfile, "}\n#endif /* unix */\n\n\n");

/*---------------------------------------------------------------------------
    Write other() function.
  ---------------------------------------------------------------------------*/

    fprintf(outfile, "other()\n{\n");
    fprintf(outfile, "    printf(\"\\nAdditionally, the following standard tokens are defined:\\n\");\n\n");

    for (i = 0;  i < sizeof(standard)/sizeof(char *);  ++i) {
        fprintf(outfile, "#ifdef %s\n", standard[i]);
        fprintf(outfile, "    printf( COMPARE(sizeof(%s),sizeof(int)) ?\n", 
          standard[i]);
        fprintf(outfile, "      \"%24s (%%d)\\n\" :\n", standard[i]);
        fprintf(outfile, "      \"%24s (%%s)\\n\", %s);\n", standard[i], 
          standard[i]);
        fprintf(outfile, "#endif\n");
    }

    fprintf(outfile, "}\n");

/*---------------------------------------------------------------------------
    All done.
  ---------------------------------------------------------------------------*/

    fclose(outfile);
    printf("\tProgram defind.c created (%d+%d possible tokens).\n", 
      num_tokens, i);
    exit(0);
}
